@@ -320,6 +320,8 @@ class Runner:
         self._cancelled = False
         self._proc = None
         self._tasks = []
+        self._timer = None  # asyncio.TimerHandle
+        self._timed_out = False  # True if runner timeout expired
 
     @property
     def name(self):
@@ -366,7 +368,7 @@ class Runner:
             self._options.encoding,
         )
 
-        return make_result(self.command, result, self._cancelled)
+        return make_result(self.command, result, self._cancelled, self._timed_out)
 
     def add_task(self, coro, tag=""):
         "Add a background task."
@@ -392,7 +394,7 @@ class Runner:
 
         except asyncio.CancelledError:
             LOGGER.info("Runner.wait cancelled %r", self)
-            self._cancelled = True
+            self._set_cancelled()
             self._tasks.clear()  # all tasks were cancelled
             await self._kill()
 
@@ -403,6 +405,11 @@ class Runner:
         while True:
             status = wait_pid(self._proc.pid)
             if status is not None:
+                LOGGER.debug(
+                    "process %r exited with returncode %r (wait_pid)",
+                    self._proc.pid,
+                    status,
+                )
                 # pylint: disable=protected-access
                 self._proc._transport._returncode = status
                 self._proc._transport._proc.returncode = status
@@ -432,7 +439,7 @@ class Runner:
         except (asyncio.CancelledError, asyncio.TimeoutError) as ex:
             LOGGER.warning("Runner.kill %r (ex)=%r", self, ex)
             if _is_cancelled(ex):
-                self._cancelled = True
+                self._set_cancelled()
             await self._kill_wait()
 
         except Exception as ex:
@@ -474,6 +481,7 @@ class Runner:
         try:
             return await self._start()
         except BaseException as ex:
+            self._stop_timer()  # failsafe just in case
             self._audit_callback("stop", failure=type(ex).__name__)
             raise
         finally:
@@ -517,7 +525,7 @@ class Runner:
         except (Exception, asyncio.CancelledError) as ex:
             LOGGER.info("Runner._start %r ex=%r", self, ex)
             if _is_cancelled(ex):
-                self._cancelled = True
+                self._set_cancelled()
             if self._proc:
                 await self._kill()
             raise
@@ -532,6 +540,9 @@ class Runner:
         if not self._is_bsd_pty():
             self.add_task(self._waiter(), "waiter")
 
+        # Set a timer to cancel the current task after a timeout.
+        self._start_timer(self.command.options.timeout)
+
         return self
 
     @log_method(LOG_DETAIL)
@@ -541,14 +552,13 @@ class Runner:
         # Second half of pty setup.
         if opts.pty_fds:
             opts.pty_fds = await opts.pty_fds.open_streams()
-            if _BSD:
-                pty_util.patch_child_watcher()
 
         # Check for task cancellation and yield right before exec'ing. If the
         # current task is already cancelled, this will raise a CancelledError,
         # and we save ourselves the work of launching and immediately killing
         # a process.
         await asyncio.sleep(0)
+
         # Launch the subprocess (always completes even if cancelled).
         await uninterrupted(self._subprocess_exec(opts))
 
@@ -561,6 +571,8 @@ class Runner:
         "Start the subprocess and assign to `self.proc`."
         with log_timer("asyncio.create_subprocess_exec"):
             sys.audit(AUDIT_EVENT_SUBPROCESS_SPAWN, opts.args[0])
+            if opts.pty_fds and _BSD:
+                pty_util.patch_child_watcher()
             self._proc = await asyncio.create_subprocess_exec(
                 *opts.args,
                 **opts.kwd_args,
@@ -569,10 +581,40 @@ class Runner:
     @log_method(LOG_DETAIL)
     async def _waiter(self):
         "Run task that waits for process to exit."
-        if self._is_bsd_pty():
-            await self._wait_pid()
-        else:
-            await self._proc.wait()
+        try:
+            if self._is_bsd_pty():
+                await self._wait_pid()
+            else:
+                await self._proc.wait()
+        finally:
+            self._stop_timer()
+
+    def _set_cancelled(self):
+        "Set the cancelled flag, and cancel any inflight timers."
+        self._cancelled = True
+        self._stop_timer()
+
+    def _start_timer(self, timeout):
+        "Start an optional timer to cancel the process if `timeout` desired."
+        assert self._timer is None
+        if timeout is not None:
+            loop = asyncio.get_running_loop()
+            self._timer = loop.call_later(
+                timeout,
+                self._set_timer_expired,
+                asyncio.current_task(),
+            )
+
+    def _set_timer_expired(self, main_task):
+        "Set a flag when the timer expires and cancel the main task."
+        self._timed_out = True
+        self._timer = None
+        main_task.cancel()
+
+    def _stop_timer(self):
+        if self._timer:
+            self._timer.cancel()
+            self._timer = None
 
     def _setup_input_source(self, stream, opts):
         "Set up a task to read from custom input source."
@@ -629,9 +671,17 @@ class Runner:
             suppress = await self._finish(exc_value)
         except asyncio.CancelledError:
             LOGGER.info("Runner cancelled inside _finish %r", self)
-            self._cancelled = True
+            self._set_cancelled()
         finally:
+            self._stop_timer()  # failsafe just in case
             self._audit_callback("stop")
+        # If `timeout` expired, raise TimeoutError rather than CancelledError.
+        if (
+            self._cancelled
+            and self._timed_out
+            and not self.command.options.incomplete_result
+        ):
+            raise asyncio.TimeoutError()
         return suppress
 
     @log_method(LOG_DETAIL)
@@ -642,7 +692,7 @@ class Runner:
         try:
             if exc_value is not None:
                 if _is_cancelled(exc_value):
-                    self._cancelled = True
+                    self._set_cancelled()
                 await self._kill()
                 return self._cancelled
 
